
from rest_framework import viewsets
from rest_framework import serializers

from .pagination import Pagination


def get_readonly_view(model_class):

    _ = type(f'{model_class.__name__}_AutoSerializer', (serializers.ModelSerializer,), dict(
        Meta=type('Meta', (object,), dict(
            model=model_class,
            fields='__all__',
        ))
    ))

    class ReadonlyModelView(viewsets.ReadOnlyModelViewSet):
        queryset = model_class.objects.all()
        pagination_class = Pagination
        serializer_class = _

    return ReadonlyModelView




from django.db.models import Count

from rest_framework.decorators import action
from rest_framework.response import Response


class AutocompleteTextFieldMixin:

    @action(detail=False)
    def autocomplete_text_field(self, request):
        field = request.GET.get('field')
        qs = self.get_queryset().values(field).filter(**{
            f'{field}__isnull': False
        }).annotate(
            num=Count(field)
        ).order_by(field)
        return Response([
            item[field] for item in qs
        ])




from django.utils import timezone

from rest_framework.response import Response
from rest_framework import pagination


class Pagination(pagination.PageNumberPagination):

    page_size = 100
    max_page_size = 100
    page_size_query_param = 'page_size'

    def get_paginated_response(self, data, **kwargs):
        return Response(dict(
            total=self.page.paginator.count,
            timestamp=timezone.now().timestamp(),
            pagination=dict(
                next=self.page.next_page_number() if self.page.has_next() else None,
                previous=self.page.previous_page_number() if self.page.has_previous() else None,
                next_link=self.get_next_link(),
                previous_link=self.get_previous_link(),
                per_page=self.page.paginator.per_page,
                page=self.page.number,
                total_pages=self.page.paginator.num_pages
            ),
            results=data,
            **kwargs
        ))


def get_pagination_class(**kwargs):
    return type('AutoPagination', (Pagination,), kwargs)




import django_filters

from django import forms
from django.utils.translation import ugettext_lazy as _


class NullBooleanSelect(forms.Select):
    def __init__(self, attrs=None):
        choices = (
            ('false', _('No')),
            ('true', _('Yes')),
            ('', _('All')),
        )
        super().__init__(attrs, choices)

    def format_value(self, value):
        try:
            return {True: 'true', False: 'false'}[value]
        except KeyError:
            return ''

    def value_from_datadict(self, data, files, name):
        value = data.get(name)
        return {
            'true': True,
            True: True,
            'false': False,
            False: False,
        }.get(value)


class NullBooleanField(forms.NullBooleanField):
    widget = NullBooleanSelect


class NullBooleanFilter(django_filters.BooleanFilter):
    field_class = NullBooleanField




from django.db.models.fields.related import ManyToManyField
from django.db.models import FieldDoesNotExist

__all__ = [
    'RepresentationMixin'
]


class RepresentationMixin:
    related_representations = {}

    def to_representation(self, instance):
        data = super().to_representation(instance)
        model_meta = self.Meta.model._meta
        for field_name, serializer_class in self.related_representations.items():
            data[field_name] = None
            related_instance = None
            many = False

            try:
                field = model_meta.get_field(field_name)
                if isinstance(field, (ManyToManyField,)):
                    related_instance = getattr(instance, field_name).all()
                    many = True
            except FieldDoesNotExist:
                pass

            if related_instance is None:
                related_instance = getattr(instance, field_name)

            if related_instance is not None:
                data[field_name] = serializer_class(instance=related_instance, context=self.context, many=many).data
        return data




from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist

from rest_framework import fields
from rest_framework.relations import PrimaryKeyRelatedField
from rest_framework import serializers


class ObjectPrimaryKeyRelatedField(PrimaryKeyRelatedField):

    def to_internal_value(self, data):
        if isinstance(data, (dict,)):
            data = data.get('id', None)
        return super().to_internal_value(data)


class StringRepresentationPrimaryKeyRelatedField(PrimaryKeyRelatedField):

    def use_pk_only_optimization(self):
        return False

    def to_representation(self, value):
        return value.__str__()


class ObjectMetaField(fields.Field):

    def __init__(self, *args, **kwargs):
        super().__init__(read_only=True, source='*')

    def to_representation(self, value):
        meta = value._meta
        cls = value.__class__
        data = {
            'verbose_name': meta.verbose_name,
            'verbose_name_plural': meta.verbose_name_plural,
            'app_label': meta.app_label,
            'model_name': meta.model_name,
            'model': f'{meta.app_label}.{meta.model_name}',
            'content_type_id': ContentType.objects.get_for_model(value).id
        }

        if hasattr(value, 'api_detail_url'):
            data['url'] = value.api_detail_url

        if hasattr(cls, 'api_list_url'):
            data['list_url'] = cls.api_list_url()

        return data

    def to_internal_value(self, data):
        return None


class DisplayField(fields.Field):

    def __init__(self, *args, **kwargs):
        super().__init__(read_only=True, source='*')

    def to_representation(self, value):
        if hasattr(value, 'display'):
            return getattr(value, 'display')

        return value.__str__()

    def to_internal_value(self, data):
        return None


class ContentTypeField(serializers.PrimaryKeyRelatedField):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def get_queryset(self):
        return ContentType.objects.all()

    def to_internal_value(self, data):
        search = {}
        if isinstance(data, (str,)) and data.count('.') == 1:
            app_label, model = data.split('.')
            search.update({
                'app_label': app_label,
                'model': model
            })
        else:
            search = {
                'pk': data
            }

        try:
            return self.get_queryset().get(**search)
        except ObjectDoesNotExist:
            self.fail('does_not_exist', pk_value=data)
        except (TypeError, ValueError):
            self.fail('incorrect_type', data_type=type(data).__name__)




from rest_framework_extensions.key_constructor.constructors import DefaultListKeyConstructor


class ListKeyConstructor(DefaultListKeyConstructor):

    def get_key(self, view_instance, view_method, request, args, kwargs):
        key = super().get_key(view_instance, view_method, request, args, kwargs)
        meta = view_instance.queryset.model._meta
        return f'{meta.app_label}.{meta.object_name}.{key}'


list_cache_key_func = ListKeyConstructor()



import setuptools
from os.path import join, dirname

import smpl_drf

setuptools.setup(
    name='smpl-drf',
    packages=setuptools.find_packages(),
    version=smpl_drf.__version__,
    license='MIT',
    url='https://github.com/ilya-muhortov/smpl-drf',
    long_description=open(join(dirname(__file__), 'README.md')).read(),
    long_description_content_type="text/markdown"
)




from rest_framework.permissions import DjangoModelPermissions


class DjangoModelWithViewPermissions(DjangoModelPermissions):

    perms_map = {
        'GET': ['%(app_label)s.view_%(model_name)s'],
        'OPTIONS': [],
        'HEAD': [],
        'POST': ['%(app_label)s.add_%(model_name)s'],
        'PUT': ['%(app_label)s.change_%(model_name)s'],
        'PATCH': ['%(app_label)s.change_%(model_name)s'],
        'DELETE': ['%(app_label)s.delete_%(model_name)s'],
    }
